%% Function gen_rand   
% Generates random time series, distorted by white noise, which is based on randomly generated change points
%% Input
% * nb_data - number of data points from which time series is built
% * nb_ch_pts - number of expected change points that are to be generated
% * nb_repeat - number of trials of random data generation
% * snr - Signal to Noise Ratio in dB included in white noise
%% Output
% * result - generated random time series with random change points
% * change_points - generated randomly change points used as a reference
%% Copyrights
% (C) All rights reserved.
%
% The code may be used free of charge for non-commercial and
% educational purposes, the only requirement is that this text is
% preserved within the derivative work. For any other purpose you
% must contact the authors for permission. This code may not be
% redistributed without written permission from the authors.
% 
% ABOUT:
% This software implements our approach to detect changes in multi-variate
% time series
% 
% IMPORTANT:
% If you use this software you should cite the following in any resulting publication: 
% [1] Michał Staniszewski, Agnieszka Skorupa, Łukasz Boguszewicz, Maria Sokół and Andrzej Polański. Quality Control Procedure Based on Partitioning of NMR Time Series.
%%
function [result,change_points]=gen_rand(nb_data,nb_ch_pts,nb_repeat,snr)
    offset = 10; %minimal distance between two change points
    change_points = sort(randperm(nb_data-offset, nb_ch_pts)); %random change points
    change_points = change_points(change_points>offset);
    while size(change_points,2)<nb_ch_pts %ensuring given offset
        change_points = sort(randperm(nb_data-offset, nb_ch_pts)); %random change points
        change_points = change_points(change_points>offset);
    end
    change_points = [1 change_points nb_data];
    result=zeros(nb_data-1,nb_repeat);
    segments = rand(nb_repeat,nb_ch_pts+1); %random values of mean
    %iterate over number of repeats
    for i=1:nb_repeat
        data = [];
        %iterate over number of time series
        for j=1:nb_ch_pts+1
            probe = ones(1,change_points(j+1)-change_points(j))*segments(i,j);
            data = [data probe];
        end
        try 
            data = awgn(data,snr,'measured')';
        catch ME
            switch ME.identifier
                case 'MATLAB:UndefinedFunction'
                    disp('Function awgn is undefined. Check the Communications System Toolbox.');
                    disp('Using rand() function.');
                    data = data + rand(size(data))/snr;
                otherwise
                    disp('Cannot compute the random data.')
            end
        end
        result(:,i) = data;
    end
%     plot(data)
end